import {
  authInstance,
  cityStore,
  languageStore,
  languageInstance,
  Language,
} from "@shared";
import { makeAutoObservable, runInAction } from "mobx";

export type Carrier = {
  id: number;
  short_name: string;
  full_name: string;
  slogan: string;
  city: string;
  city_id: number;
  logo: string;
  // main_color: string;
  // left_color: string;
  // right_color: string;
};

type CarrierData = {
  data: Carrier[];
  loaded: boolean;
};

type Carriers = {
  ru: CarrierData;
  en: CarrierData;
  zh: CarrierData;
};

type CashedCarrier = Record<
  number,
  {
    ru: Carrier | null;
    en: Carrier | null;
    zh: Carrier | null;
  }
>;

class CarrierStore {
  carriers: Carriers = {
    ru: {
      data: [],
      loaded: false,
    },
    en: {
      data: [],
      loaded: false,
    },
    zh: {
      data: [],
      loaded: false,
    },
  };
  carrier: CashedCarrier = {};

  constructor() {
    makeAutoObservable(this);
  }

  getCarriers = async (language: Language) => {
    if (this.carriers[language as keyof Carriers].loaded) return;

    const response = await authInstance.get("/carrier");

    runInAction(() => {
      this.carriers[language as keyof Carriers].data = response.data;
      this.carriers[language as keyof Carriers].loaded = true;
    });
  };

  deleteCarrier = async (id: number) => {
    await authInstance.delete(`/carrier/${id}`);

    runInAction(() => {
      for (const language of ["ru", "en", "zh"] as const) {
        this.carriers[language].data = this.carriers[language].data.filter(
          (carrier: Carrier) => carrier.id !== id
        );
      }
      delete this.carrier[id];
    });
  };

  getCarrier = async (id: number) => {
    if (this.carrier[id]?.ru && this.carrier[id]?.en && this.carrier[id]?.zh)
      return;

    const ruResponse = await languageInstance("ru").get(`/carrier/${id}`);
    const enResponse = await languageInstance("en").get(`/carrier/${id}`);
    const zhResponse = await languageInstance("zh").get(`/carrier/${id}`);

    runInAction(() => {
      if (!this.carrier[id]) {
        this.carrier[id] = {
          ru: null,
          en: null,
          zh: null,
        };
      }
      this.carrier[id].ru = ruResponse.data;
      this.carrier[id].en = enResponse.data;
      this.carrier[id].zh = zhResponse.data;
    });
    return this.carrier[id];
  };

  createCarrier = async (
    fullName: string,
    shortName: string,
    cityId: number,
    slogan: string,
    logoId: string
  ) => {
    const { language } = languageStore;
    const cityName =
      cityStore.cities[language].data.find((city) => city.id === cityId)
        ?.name || "";

    const response = await languageInstance(language).post("/carrier", {
      full_name: fullName,
      short_name: shortName,
      city: cityName,
      city_id: cityId,
      slogan,
      logo: logoId,
    });

    const carrierId = response.data.id;

    // Create translations for other languages
    for (const lang of ["ru", "en", "zh"].filter((l) => l !== language)) {
      await languageInstance(lang as Language).patch(`/carrier/${carrierId}`, {
        full_name: fullName,
        short_name: shortName,
        city: cityName,
        city_id: cityId,
        slogan,
        logo: logoId,
      });
    }

    runInAction(() => {
      for (const language of ["ru", "en", "zh"] as const) {
        this.carriers[language].data.push(response.data);
      }
    });
  };

  editCarrierData = {
    ru: {
      full_name: "",
      short_name: "",

      // main_color: "",
      // left_color: "",
      // right_color: "",
      slogan: "",
    },
    en: {
      full_name: "",
      short_name: "",

      // main_color: "",
      // left_color: "",
      // right_color: "",
      slogan: "",
    },
    city_id: 0,
    logo: "",
    zh: {
      full_name: "",
      short_name: "",

      // main_color: "",
      // left_color: "",
      // right_color: "",
      slogan: "",
    },
  };

  setEditCarrierData = (
    fullName: string,
    shortName: string,
    cityId: number,
    // main_color: string,
    // left_color: string,
    // right_color: string,
    slogan: string,
    logoId: string,
    language: Language
  ) => {
    this.editCarrierData.city_id = cityId;
    this.editCarrierData.logo = logoId;
    this.editCarrierData[language] = {
      full_name: fullName,
      short_name: shortName,
      // main_color: main_color,
      // left_color: left_color,
      // right_color: right_color,
      slogan: slogan,
    };
  };

  editCarrier = async (id: number) => {
    const cityName =
      cityStore.cities[languageStore.language].data.find(
        (city) => city.id === this.editCarrierData.city_id
      )?.name || "";

    for (const language of ["ru", "en", "zh"] as const) {
      const response = await languageInstance(language).patch(
        `/carrier/${id}`,
        {
          ...this.editCarrierData[language],
          city: cityName,
          logo: this.editCarrierData.logo,
        }
      );

      runInAction(() => {
        if (this.carrier[id]) {
          this.carrier[id][language] = response.data;
        }
        for (const language of ["ru", "en", "zh"] as const) {
          this.carriers[language].data = this.carriers[language].data.map(
            (carrier: Carrier) =>
              carrier.id === id ? { ...carrier, ...response.data } : carrier
          );
        }
      });
    }
  };
}

export const carrierStore = new CarrierStore();
